@@ -40,6 +40,9 @@
 #include "gtklabel.h"
 #include "gtkfilechooserentry.h"
 #include "gtkfilefilterprivate.h"
+#ifdef GDK_WINDOWING_QUARTZ
+#include <gdk/quartz/gdkquartz.h>
+#endif
 
 /**
  * SECTION:gtkfilechoosernative
@@ -373,7 +376,7 @@ gtk_file_chooser_native_add_choice (GtkFileChooser  *chooser,
   choice->options = g_strdupv ((char **)options);
   choice->option_labels = g_strdupv ((char **)option_labels);
 
-  self->choices = g_slist_prepend (self->choices, choice);
+  self->choices = g_slist_append (self->choices, choice);
 
   gtk_file_chooser_add_choice (GTK_FILE_CHOOSER (self->dialog),
                                id, label, options, option_labels);
@@ -466,6 +469,7 @@ gtk_file_chooser_native_set_property (GObject      *object,
 
     case GTK_FILE_CHOOSER_PROP_FILTER:
       self->current_filter = g_value_get_object (value);
+      gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (self->dialog), self->current_filter);
       g_object_notify (G_OBJECT (self), "filter");
       break;
 
@@ -494,6 +498,7 @@ gtk_file_chooser_native_get_property (GObject    *object,
       break;
 
     case GTK_FILE_CHOOSER_PROP_FILTER:
+      self->current_filter = gtk_file_chooser_get_filter (GTK_FILE_CHOOSER (self->dialog));
       g_value_set_object (value, self->current_filter);
       break;
 
@@ -652,7 +657,9 @@ show_dialog (GtkFileChooserNative *self)
                     G_CALLBACK (dialog_update_preview_cb),
                     self);
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_window_present (GTK_WINDOW (self->dialog));
+  G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 static void
@@ -751,8 +758,10 @@ gtk_file_chooser_native_show (GtkNativeDialog *native)
     self->mode = MODE_WIN32;
 #endif
 
-#ifdef GDK_WINDOWING_QUARTZ
-  if (gtk_file_chooser_native_quartz_show (self))
+#if defined (GDK_WINDOWING_QUARTZ) && \
+  MAC_OS_X_VERSION_MAX_ALLOWED >= 1060
+    if (gdk_quartz_osx_version() >= GDK_OSX_SNOW_LEOPARD &&
+        gtk_file_chooser_native_quartz_show (self))
     self->mode = MODE_QUARTZ;
 #endif
 
@@ -780,8 +789,10 @@ gtk_file_chooser_native_hide (GtkNativeDialog *native)
 #endif
       break;
     case MODE_QUARTZ:
-#ifdef GDK_WINDOWING_QUARTZ
-      gtk_file_chooser_native_quartz_hide (self);
+#if defined (GDK_WINDOWING_QUARTZ) && \
+  MAC_OS_X_VERSION_MAX_ALLOWED >= 1060
+      if (gdk_quartz_osx_version() >= GDK_OSX_SNOW_LEOPARD)
+        gtk_file_chooser_native_quartz_hide (self);
 #endif
       break;
     case MODE_PORTAL:
